var Url = require("url");
var QueryString = require("querystring");
var Cookie = require("contentcube/cookie");
var LogCreator = require("contentcube/logger");
var Formidable = require("formidable");

// add system specific behavior to the standard `http.ServerRequest` object.
exports.decorate = function(request) {
	var mIsDispatched = false;
	var mIsDataReceived = false;
	var mParams = [];
	var mUrlParts = Url.parse(request.url);
	var mGetParams;
	var mCookieParams;
	var mPostParams;
	var mFiles;
	var mClientParams;
	var mPlaceHolders = {};
	var mLogger;
	
	if (request.method.toUpperCase() == 'POST') {
		var form = new Formidable.IncomingForm();
		form.on("end", function() {
			mIsDataReceived = true;
			request.emit("request:data_received");
		});
	
		form.parse(request, function(error, fields, files) {
			mPostParams = fields;
			mFiles = files;
		});
	}
	
	else {
		mIsDataReceived = true;
		request.on("end", function() {
			request.emit("request:data_received");
		});
	}
	
	request.isDataReceived = function() {
		return mIsDataReceived;
	}
	
	/**
	 * A mark used by the dispatcher to tell if the system i done with handling it.
	 * @param {Boolean} value
	 */
	request.setDispatched = function(value) {
		mIsDispatched = value;
	};	
	
	request.isDispatched = function() {
		return mIsDispatched;
	};
	
	/**
	 * additional parameters that will be sent to the controllers action method under execution.
	 * @param {Array} params
	 */
	request.setParams = function(params) {
		mParams = params;
	};
	
	request.getParams = function(params) {
		return mParams;
	};
	
	/**
	 * Retreive GET data by key. If no key present the whole map is returned.
	 * @param {String} key
	 * @return {Mixed}
	 */
	request.get = function(key, defaultValue) {
		if (!mGetParams) {
			mGetParams = QueryString.parse(mUrlParts.query);
		}
		if (!key) {
			return mGetParams;
		}
		
		return mGET[key] || defaultValue;
	};
	
	/**
	 * Retreive POST data by key. If no key present the whole map is returned.
	 * @param {String} key
	 * @return {Mixed}
	 */
	request.post = function(key, defaultValue) {
		if (!mPostParams) { 
			return defaultValue;
		}
		if (!key) {
			return mPostParams;
		}
		
		return mPostParams[key] || defaultValue;
	};
	
	request.files = function(key, defaultValue) {
		if (!mFiles) return false;
		if (!key) {
			return mFiles;
		}
		return mFiles[key] || defaultValue;
	};
	
	request.client = function(key, defaultValue) {
		
		if (!mClientParams) {
			mClientParams = {
				'user-agent': request.headers['user-agent'],
				'remote-address': request.connection.remoteAddress,
				'remote-port': request.connection.remotePort
			};
		}
		
		if (!key) {
			return mClientParams;
		}
		
		return mClientParams[key] || defaultValue;
	};
	
	request.cookie = function(key, defaultValue) {
		if (!mCookieParams) {
			mCookieParams = Cookie.parse(request.headers['cookie']);
		}
		if (!key) {
			return mCookieParams;
		}
		
		return mCookieParams[key] || defaultValue;
	};
	
	/**
	 * Request a placeholder, a new will be created if no match for the key
	 * @param {String} key
	 * @return {Object}
	 */
	request.placeholder = function(key) {
		if (!mPlaceHolders[key]) {
			mPlaceHolders[key] = {};
		}
		
		return mPlaceHolders[key];
	};
	
	request.session = function(key, defaultValue) {
		
	};
	
	request.getLogger = function() {
		if (!mLogger) {
			mLogger = LogCreator(request, {});
		}
		
		return mLogger;
	}
	
	return request;
};